/**
 * ==================================================================
 * CHANGE HISTORY for mmx.h
 * ------------------------------------------------------------------
 * Date       Change-id   Produced  User Name
 * yyyymmdd               Version.
 *                        rr.ss.uu
 * Reason
 * ------------------------------------------------------------------
 * 20011119   RML000      01.00.00  R. Lacroix
 * Birth.
 *
 *
 * ------------------------------------------------------------------
 */
#ifndef __MRTRDEF
   #define __MRTRDEF 1 /* Indicate that this file has been included. */

/* ------------------------------------------------------------------
 * The following defines can be turned on for use during testing
 * and turn off when not required.  Use the compiler flag DIAGNOSE
 * to enable the output of the TEST_ defines.
 *
 * Usage:
 *
 *    TEST_PRINT  - for simple things like "HELLO WORLD"
 *    TEST_DATA   - for formatted output of integers & strings.
 *    TEST_BUFFER - for printing large string arrays.
 *    TEST_HEX    - for printing the hexidecimals value of the buffer
 *
 *
 * Note: Do not remove the TEST_ defines from the code, just remove
 *       the compiler switch DIAGNOSE to turn off the output.
 *
 * ------------------------------------------------------------------
 */
#ifdef DIAGNOSE
   #define TEST_PRINT(x) {printf (x); fflush (stdout);}

   #define TEST_DATA(a) {printf a; fflush (stdout);}

   #define TEST_BUFFER(TITLE,BUFFER,LENGTH) {int i;printf(TITLE);    \
                for(i=0;i<LENGTH;                                    \
                   printf("%c",*( ((char *)BUFFER) + (i++) )));      \
                printf("\n"); fflush (stdout); }

   #define TEST_HEX(TITLE,BUFFER,LENGTH) {int i;printf(TITLE);       \
                for(i=0;i<LENGTH;                                    \
                   printf("%1.1x",*( ((char *)BUFFER) + (i++) )));   \
                printf("\n"); fflush (stdout); }
#else
   #define TEST_PRINT(x)
   #define TEST_DATA(a)
   #define TEST_BUFFER(x,y,z)
   #define TEST_HEX(x,y,z)
#endif

/**
 * Compile-time Macro to determine the length of a
 * string without the null terminator.
 */
#define STRING_SIZE(x) ( sizeof( x) - 1 )

/**
 * General Purpose defines
 */
#ifdef MVS
/* format is:  DD:NameOfDD */
#define _MAX_PATH                      13
/* max length of MVS sysin */
#define MAX_INI_LINE_LENGTH            80
#else
/* max for distributed plat */
#define MAX_INI_LINE_LENGTH            256
#endif


#if defined(UNIX) || defined(OS400)

#ifndef PATH_MAX
#define PATH_MAX                       256
#endif
/* Unix max path length */
#define _MAX_PATH                      PATH_MAX
#endif


#define NULL_STRING                    ""
/* string equal indicator */
#define STR_EQ                         0

/* Max message size when dealing with OS/390 (MVS) */
#define MAX_MESSAGE_LENGTH             4194304

/* */
#if defined(WIN32)
#define HOSTED_PLATFORM              'W'
#define HOSTED_PLATFORM_NAME         "Windows"

#elif SUN
#define HOSTED_PLATFORM              'S'
#define HOSTED_PLATFORM_NAME         "Solaris"

#elif LINUX
#define HOSTED_PLATFORM              'L'
#define HOSTED_PLATFORM_NAME         "Linux"

#elif AIX
#define HOSTED_PLATFORM              'A'
#define HOSTED_PLATFORM_NAME         "AIX"

#elif HPUX
#define HOSTED_PLATFORM              'H'
#define HOSTED_PLATFORM_NAME         "HPUX"

#elif defined(OS400)
#define HOSTED_PLATFORM              'I'
#define HOSTED_PLATFORM_NAME         "iSeries"

#elif TANDEM
#define HOSTED_PLATFORM              'T'
#define HOSTED_PLATFORM_NAME         "Tandem"

#elif defined(MVS)
#define HOSTED_PLATFORM              'Z'
#define HOSTED_PLATFORM_NAME         "zOS"

#endif

/*
 * defines used for Log functions
 */
#define LOG_TYPE_INFO         0   /* General informational log msg */
#define LOG_TYPE_WARNING      1   /* Warning messages */
#define LOG_TYPE_ERROR        2   /* Error messages - show stoppers */

/**
 * Initialization file entries to be search for relating to MMX.
 */
#define INI_TIMEOUT                    "TimeOut"
#define INI_LOGFILE                    "LogFile"
#define INI_SEQUENCE_NUMBER_FLAG       "SequenceNumberFlag"
#define INI_PERFORM_LOGGING            "PerformLogging"
#define INI_MSG_BUFFER_SIZE            "GetBufferSize"
#define INI_QMGR_NAME                  "QMgrName"
#define INI_INPUT_Q_NAME               "InputQName"
#define INI_GET_BUFFER_SIZE            "GetBufferSize"
#define INI_START_KEYWORD              "StartKeyword"
#define INI_END_KEYWORD                "EndKeyword"
#define INI_DEAD_LETTER_QUEUE          "RouterDeadQ"
#define INI_GET_WITH_CONVERT           "GetWithConvert"

/**
 * Default values for initialization entries relating to MRTR.
 */
#ifdef MVS
#define DEFAULT_LOGFILE                    "SYSPRINT"
#else
#define DEFAULT_LOGFILE                    "mrtr.log"
#endif
#define DEFAULT_SEQUENCE_NUMBER_FLAG          "N"
#define DEFAULT_TIMEOUT_VALUE                 10
#define DEFAULT_PERFORM_LOGGING               "N "
#define DEFAULT_QMGR_NAME                     " "
#define DEFAULT_INPUT_Q_NAME                  ""
#define DEFAULT_OUTPUT_Q_COUNT                 0
#define DEFAULT_OUTPUT_Q_NAME                 ""
#define DEFAULT_MSG_BUFFER_SIZE              65536
#define DEFAULT_START_KEYWORD                 ""
#define DEFAULT_END_KEYWORD                   ""
#define DEFAULT_DEAD_LETTER_QUEUE             "IA.DeadQ"
#define DEFAULT_INI_NO                        "N"
#define DEFAULT_INI_YES                       "Y"
#define DEFAULT_INI_BLANK                     ""


/**
 * sprintf define for logging
 */
#define LOG_RECORD_FORMAT_STRING "%d/%02d/%02d %02d:%02d:%02d %s "\
"#%0.5d: "

/**
 * Global structure (work area)
 */
typedef struct MRTR_GLOBAL_S
{
   int         ConnectedFlag          ; /* flag for connected status */
   int         OpenedInputQueue       ; /* flag for opened queue */
   int         MessageBufferSize      ; /* size of the buffer */
   int         TotalMsgRead           ; /* Count of Messages read */
   int         TotalMsgWritten        ; /* Count of Messages written */
   int         TimeOut                ; /* mqget timeout value */
   MQLONG      ActualMessageSize      ; /* actual size of msg data */

   MQHCONN     Hconn                  ; /* connection handle */
   MQHOBJ      HobjInQ                ; /* object handle */

   char       *pIMB                   ; /* ptr Input Msg Buffer */
   char       *pOMB                   ; /* prt Output Msg Buffer */

   char        GetWithConvert[2]      ; /*  [Y/N] */
   char        PerformLogging[2]      ; /* do logging? [Y/N] */
   char        SequenceNumberFlag     ; /* ini file have sequence# */

   MQCHAR      QMgrName[MQ_Q_MGR_NAME_LENGTH];/* queue manager name */
   MQCHAR      InputQName[MQ_Q_NAME_LENGTH];  /* input queue name */
   MQCHAR      OutputQName[MQ_Q_NAME_LENGTH]; /* output queue name */
   MQCHAR      RouterDeadQName[MQ_Q_NAME_LENGTH]; /* output q name */
   MQMD        OriginalMD             ; /* */
   char        iniFile[_MAX_PATH]     ; /* ini file */
   char        LogFile[_MAX_PATH]     ; /* log file */
   char        StartKeyword[128]      ; /* Beginning keyword */
   char        EndKeyword[128]        ; /* Ending keyword */
   char        KeywordValue[128]      ; /* value for keyword */
   char        TempBuffer[512]        ; /* work area */

} MRTR_GLOBAL_T;

/**
 * Function Protypes
 */
int  Initialize( void );
void Terminate( void );
void LogIt( int, int, char [] );
void RemovePrefixBlanks( char * );
void RemoveTrailingBlanks( char * );
int  GetCmdLineParms(int argc, char *argv[]);
int  GetInitFileParameters( void );
void Print_MRTR_Global( void );
void PrintStatistics( void );
int  ProcessMessages( void );
int  ReadMessage( void );
void FindKeyword( void );
int  PutMessage( MQCHAR * );
int  CommitMessages( void );
int  BackOutMessages( void );
int  GetProfileInt( char *, int,    char *, char );
int  GetProfileString( char *, char *, char *, int, char *,
                       char );
int  ReadLine( FILE *, char  *, char );
int  CalculateMemorySize( int );

int  ConnectToMqm( void );
int  DisconnectFromMqm( void );
int  OpenInputQueue( void );
int  CloseInputQueue( void );

#endif  /* End of header file */
